"""Default description (change me)"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/client/MafiaError.ipynb.

# %% auto 0
__all__ = ['generate_error_message', 'MafiaError']

# %% ../../nbs/client/MafiaError.ipynb 3
def generate_error_message(message=None, exception=None):

    if exception:
        if message == None:
            message = str(exception)

        template = f"An exception of type {type(exception) and type(exception).__name__ or 'unknown'} occurred."

        if exception.args:
            template += f" Arguments: {','.join(str(arg) for arg in exception.args if arg and str(arg))}"

        message = f"{message}\n{template}"

    if not message.startswith("💀"):
        message = "💀  " + message

    return message

# %% ../../nbs/client/MafiaError.ipynb 4
class MafiaError(Exception):
    def __init__(self, message=None, exception=None):

        super().__init__(generate_error_message(message=message, exception=exception))



"""Default description (change me)"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/utils/chunking.ipynb.

# %% auto 0
__all__ = ['default_calc_end_fns', 'calc_end_codeblock', 'calc_end_paragraph', 'calc_end_sentence', 'functions_stop_on_value',
           'chunk_text']

# %% ../../nbs/utils/chunking.ipynb 3
from typing import Union, List, Callable, Union

# %% ../../nbs/utils/chunking.ipynb 4
def calc_end_codeblock(chunk, chunk_size, start) -> Union[int, bool]:
    code_block = chunk.rfind("```")
    if code_block == -1 or code_block <= chunk_size * 0.3:
        return False

    return start + code_block

# %% ../../nbs/utils/chunking.ipynb 5
def calc_end_paragraph(chunk, chunk_size, start) -> Union[int, bool]:
    last_break = chunk.rfind("\n\n")

    if last_break == -1 or last_break <= chunk_size * 0.3:
        return False

    return start + last_break

# %% ../../nbs/utils/chunking.ipynb 6
def calc_end_sentence(chunk, chunk_size, start) -> Union[int, bool]:
    last_period = chunk.rfind(". ")

    if last_period == -1 or last_period <= chunk_size * 0.3:
        return False

    return start + last_period + 1

# %% ../../nbs/utils/chunking.ipynb 7
def functions_stop_on_value(functions: List[Callable], **kwargs):
    """iterate over a list of functions and return the first function that returns a value"""

    return next((fn(**kwargs) for fn in functions if fn(**kwargs)), None)

# %% ../../nbs/utils/chunking.ipynb 8
default_calc_end_fns = [calc_end_codeblock, calc_end_paragraph, calc_end_sentence]

# %% ../../nbs/utils/chunking.ipynb 9
def chunk_text(
    text: str,  # text to chunk
    calc_end_fns: Union[
        List[Callable], None
    ] = None,  # list of functions to calculate the end of a chunk
    chunk_size: int = 5000,
    debug_prn: bool = False,
) -> List[str]:

    calc_end_fns = calc_end_fns or default_calc_end_fns
    chunks = []
    start = 0
    text_length = len(text)

    while start < text_length:
        end = start + chunk_size

        if end >= text_length:
            chunks.append(text[start:].strip())
            break

        # handle code block
        chunk = text[start:end]

        end = (
            functions_stop_on_value(
                functions=calc_end_fns, chunk=chunk, chunk_size=chunk_size, start=start
            )
            or end
        )

        chunk = text[start:end].strip()

        if chunk:
            chunks.append(chunk)

        start = max(start + 1, end)

    if debug_prn:
        print(
            f"Chunked {len(text)} character text into {len(chunks)} chunks of chunk_size {chunk_size}"
        )
    return chunks



"""Default description (change me)"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/utils/files.ipynb.

# %% auto 0
__all__ = ['upsert_folder', 'read_md_from_disk']

# %% ../../nbs/utils/files.ipynb 2
import os
from frontmatter import Frontmatter
import shutil

# %% ../../nbs/utils/files.ipynb 3
def upsert_folder(folder_path: str, debug_prn: bool = False, replace_folder=False):

    folder_path = os.path.dirname(folder_path)

    if replace_folder and os.path.exists(folder_path) and os.path.isdir(folder_path):
        folder_path = os.path.join(folder_path, "")
        shutil.rmtree(folder_path)

    if debug_prn:
        print(
            {
                "upsert_folder": os.path.abspath(folder_path),
                "is_exist": os.path.exists(folder_path),
            }
        )

    if not os.path.exists(folder_path):
        os.makedirs(folder_path)

# %% ../../nbs/utils/files.ipynb 4
def read_md_from_disk(file_path):
    data = Frontmatter.read_file(file_path)

    return data["body"], data["attributes"]



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/agent-mafia',
                'doc_host': 'https://jaewilson07.github.io',
                'git_url': 'https://github.com/jaewilson07/agent-mafia',
                'lib_path': 'agent_mafia'},
  'syms': { 'agent_mafia.client.MafiaError': { 'agent_mafia.client.MafiaError.MafiaError': ( 'client/mafiaerror.html#mafiaerror',
                                                                                             'agent_mafia/client/MafiaError.py'),
                                               'agent_mafia.client.MafiaError.MafiaError.__init__': ( 'client/mafiaerror.html#mafiaerror.__init__',
                                                                                                      'agent_mafia/client/MafiaError.py'),
                                               'agent_mafia.client.MafiaError.generate_error_message': ( 'client/mafiaerror.html#generate_error_message',
                                                                                                         'agent_mafia/client/MafiaError.py')},
            'agent_mafia.core': {'agent_mafia.core.foo': ('core.html#foo', 'agent_mafia/core.py')},
            'agent_mafia.utils.chunking': { 'agent_mafia.utils.chunking.calc_end_codeblock': ( 'utils/chunking.html#calc_end_codeblock',
                                                                                               'agent_mafia/utils/chunking.py'),
                                            'agent_mafia.utils.chunking.calc_end_paragraph': ( 'utils/chunking.html#calc_end_paragraph',
                                                                                               'agent_mafia/utils/chunking.py'),
                                            'agent_mafia.utils.chunking.calc_end_sentence': ( 'utils/chunking.html#calc_end_sentence',
                                                                                              'agent_mafia/utils/chunking.py'),
                                            'agent_mafia.utils.chunking.chunk_text': ( 'utils/chunking.html#chunk_text',
                                                                                       'agent_mafia/utils/chunking.py'),
                                            'agent_mafia.utils.chunking.functions_stop_on_value': ( 'utils/chunking.html#functions_stop_on_value',
                                                                                                    'agent_mafia/utils/chunking.py')},
            'agent_mafia.utils.files': { 'agent_mafia.utils.files.read_md_from_disk': ( 'utils/files.html#read_md_from_disk',
                                                                                        'agent_mafia/utils/files.py'),
                                         'agent_mafia.utils.files.upsert_folder': ( 'utils/files.html#upsert_folder',
                                                                                    'agent_mafia/utils/files.py')}}}
